 /*
  Created by Fabrizio Di Vittorio (fdivitto2013@gmail.com) - www.fabgl.com
  Copyright (c) 2019 Fabrizio Di Vittorio.
  All rights reserved.

  This file is part of FabGL Library.

  FabGL is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  FabGL is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with FabGL.  If not, see <http://www.gnu.org/licenses/>.
 */

/* 
 MH: Modifications by Martin Hepperle, July 2020 

 TTGO VGA32: programming as ESP32 Pico Kit
 */

#include "fabgl.h"

/* MH: use TekTerminal instead of Terminal */
#include "tekterminal.h"

/* MH: German keyboard layout */
#define GERMAN
/* MH: my "stabndard" baud rate */
#define BAUD_RATE   9600


#ifdef GERMAN
#include <devdrivers/keyboard.h>
#endif

fabgl::VGAController VGAController;
fabgl::PS2Controller PS2Controller;
/* MH: use TekTerminal instead of Terminal */
fabgl::TekTerminal      Terminal;


void setup()
{
  // for ESP32-PICO-D4 use GPIO 12 (RX) and 2 (TX)
//  Serial.begin(BAUD_RATE, SERIAL_8N1, 12, 2);
  // for TTGO-VGA32 use GPIO 14 (RX) and 2 (TX) [GPIO12 blocks boot process]
  Serial.begin(BAUD_RATE, SERIAL_8N1, 14, 2);

  /* MH: to be sure that Serial is instantiated and initialized */
  while ( ! Serial )
    ;

 
  // other boards
  //Serial2.begin(115200);

  //Serial.begin(115200); // debug only

  // only keyboard configured on port 0
  PS2Controller.begin(PS2Preset::KeyboardPort0);  

  /* MH: switched language to German */
#ifdef GERMAN
  PS2Controller.keyboard()->setLayout(&fabgl::GermanLayout);
#endif

  VGAController.begin();
  VGAController.setResolution(VGA_640x350_70HzAlt1, 640, 350);

  Terminal.begin(&VGAController);
  Terminal.setTerminalType  ( TermType::ANSI_VT  );   

  /* MH: set XONOFF to false, otherwise terminal sends XON when connecting */
  Terminal.connectSerialPort(Serial,false);
  
  //Terminal.setLogStream(Serial);  // debug only

  
  Terminal.setBackgroundColor(Color::Black);
  Terminal.setForegroundColor(Color::Green);
  Terminal.clear();
  Terminal.enableCursor(true);

  Terminal.write("* * FabGL - Serial ANSI+Tektronix Terminal\r\n");
  Terminal.write("* * 2019 by Fabrizio Di Vittorio - www.fabgl.com\r\n");
  Terminal.write("* * 2020 by Martin Hepperle\r\n\n");
  Terminal.printf("Screen Size        : %d x %d\r\n", VGAController.getScreenWidth(), VGAController.getScreenHeight());
  Terminal.printf("Viewport Size      : %d x %d\r\n", VGAController.getViewPortWidth(), VGAController.getViewPortHeight());
  Terminal.printf("Terminal Size      : %d x %d\r\n", Terminal.getColumns(), Terminal.getRows());
  Terminal.printf("Keyboard           : %s\r\n\r\n", PS2Controller.keyboard()->isKeyboardAvailable() ? "OK" : "Error");
  Terminal.printf("Free DMA Memory    : %d\r\n", heap_caps_get_free_size(MALLOC_CAP_DMA));
  Terminal.printf("Free 32 bit Memory : %d\r\n\n", heap_caps_get_free_size(MALLOC_CAP_32BIT));
  Terminal.write("Connect server to GPIO12 (RX) and GPIO2 (TX) - 8N1 - 9600 baud\r\n\n");
  Terminal.write((char)7);

  Terminal.setForegroundColor(Color::BrightGreen);  
}


void loop()
{
  Terminal.pollSerialPort();
}
