/*
  Created by Fabrizio Di Vittorio (fdivitto2013@gmail.com) - <http://www.fabgl.com>
  Copyright (c) 2019 Fabrizio Di Vittorio.
  All rights reserved.

  This file is part of FabGL Library.

  FabGL is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  FabGL is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with FabGL.  If not, see <http://www.gnu.org/licenses/>.
 */

 
#pragma once



/**
 * @file
 *
 * @brief This file contains fabgl::Terminal definition.
 */



#include "terminal.h"
#include <devdrivers/soundgen.h>


namespace fabgl {

class TekTerminal : public Terminal {

public:

	void soundBell();
	void eraseGraphics();
	void setTextMode();
	void setGraphicsMode( int mode );
	void tekChar ( int * pX, int * pY, char c, int fontsize );

	void consumeInputQueue();
	void execCtrlCode(char c);
	void consumeESC();
	void consumeDECPrivateModes(int const * params, int paramsCount, char c);

private:

	/* -MH- added sound for ^G BELL */
	SoundGenerator  soundGenerator;
	SineWaveformGenerator sine;

	/* -MH- added for Tektronix */
	#define MODE_TEXT    0   // text terminal mode
	#define MODE_ESC     1
	#define MODE_ALPHA   2   // alpha mode
	#define MODE_POINTS  3   // point plot
	#define MODE_GRAPH   4   // graph + pen up
	#define MODE_INC     5   // incremental plot  
	#define MODE_GIN     6   // crosshairs
	#define MODE_SPECIAL 7   // special point plot

	int x_0, y_0;
	int xColumnStart;        // left border of Tektronix 2 column alpha mode outpout
	int extra;
	int loX, hiX;
	int loY, hiY;
	int cellwidth  = 55;
	int cellheight = 117;

	boolean inESC = false;
	int mode = MODE_TEXT;
	byte seq = 0;
	byte pen = 0;  // UP
	boolean getExtra = false;
	RGB888 intensity = RGB888(0,0xFF,0);
	boolean grabIntensity = false;

};


} // end of namespace

